import numpy as np
from skimage.measure import label, regionprops_table


def mask_largest_object(
        img: np.ndarray,
        max_allowed: int = 10,
        verbose: bool = True
) -> np.ndarray:
    """
    Where more than one connected component is found in an image, return the largest object by area
    :param img: (np.ndarray) containing object labels
    :param max_allowed: raise an error if more than this number of objects is found
    :param verbose: print a message each time more than one object is found
    :return: np.ndarray of same size as img
    """
    binary = img > 0
    ob_id = label(binary)
    num_obj = len(np.unique(ob_id)) - 1
    if num_obj > max_allowed:
        raise TooManyObjectError(f'Found {num_obj} objects in frame')
    if num_obj > 1:
        if verbose:
            print(f'Found {num_obj} nonzero unique values in object map; keeping the one with the largest area')
        pr = regionprops_table(ob_id, properties=['label', 'area'])
        idx_max_area = pr['area'].argmax()
        mask = ob_id == pr['label'][idx_max_area]
        return mask * img
    else:
        return img


class Error(Exception):
    pass


class TooManyObjectError(Exception):
    pass
