from fastapi import APIRouter, HTTPException

from model_server.session import Session
from model_server.validators import  validate_workflow_inputs

from extensions.ilastik.models import IlastikImageToImageModel, IlastikPixelClassifierModel, IlastikObjectClassifierFromPixelPredictionsModel
from model_server.models import ParameterExpectedError
from extensions.ilastik.workflows import infer_px_then_ob_model

router = APIRouter(
    prefix='/ilastik',
    tags=['ilastik'],
)

session = Session()

def load_ilastik_model(model_class: IlastikImageToImageModel, project_file: str, duplicate=True) -> dict:
    """
    Load an ilastik model of a given class and project filename.
    :param model_class:
    :param project_file: (*.ilp) ilastik project filename
    :param duplicate: load another instance of the same project file if True; return existing one if false
    :return: dictionary with single key describing model's ID
    """
    if not duplicate:
        existing_model = session.find_param_in_loaded_models('project_file', project_file)
        if existing_model is not None:
            return existing_model
    try:
        result = {
            'model_id': session.load_model(
                model_class,
                {'project_file': project_file}
            )
        }
    except (FileNotFoundError, ParameterExpectedError):
        raise HTTPException(
            status_code=404,
            detail=f'Could not load project file {project_file}',
        )
    return result

@router.put('/pixel_classification/load/')
def load_ilastik_pixel_classification_model(project_file: str, duplicate: bool = True) -> dict:
    return load_ilastik_model(IlastikPixelClassifierModel, project_file, duplicate=duplicate)

@router.put('/object_classification/load/')
def load_ilastik_object_classification_model(project_file: str, duplicate: bool = True) -> dict:
    return load_ilastik_model(IlastikObjectClassifierFromPixelPredictionsModel, project_file, duplicate=duplicate)

@router.put('/pixel_then_object_classification/infer')
def infer_px_then_ob_maps(px_model_id: str, ob_model_id: str, input_filename: str, channel: int = None) -> dict:
    inpath = session.paths['inbound_images'] / input_filename
    validate_workflow_inputs([px_model_id, ob_model_id], [inpath])
    try:
        record = infer_px_then_ob_model(
            inpath,
            session.models[px_model_id]['object'],
            session.models[ob_model_id]['object'],
            session.paths['outbound_images'],
            channel=channel
        )
    except AssertionError:
        raise HTTPException(f'Incompatible models {px_model_id} and/or {ob_model_id}')
    return record